/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 * Henrik Gemal http://gemal.dk/
 *
 * Portions created by the Initial Developer are Copyright (C) 2001-2005
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var LaunchyContext = function() {}

LaunchyContext.prototype.doInit = function() {
	// did Launchy already load?
	var ass = Components.classes["@mozilla.org/appshell/appShellService;1"].getService(Components.interfaces.nsIAppShellService);
	var w = ass.hiddenDOMWindow;
	if (typeof(w.launchy) == "undefined") {
		w.launchy = LaunchyContext;
		var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		LaunchyContext.pref = prefService.getBranch(null);
		try {
			LaunchyContext.dev = LaunchyContext.pref.getBoolPref("extensions.launchy.developer");
			if (LaunchyContext.dev) {
				launchyShared.trace("Developer information is on");
			}
		} catch(ex) {
		}
		try {
			LaunchyContext.xmlfilelocation = LaunchyContext.pref.getCharPref("extensions.launchy.xmlfile");
		} catch(ex) {
		}

		// showfileicons
		try {
			LaunchyContext.showfileicons = LaunchyContext.pref.getBoolPref("extensions.launchy.showfileicons");
			if (LaunchyContext.showfileicons) {
				launchyShared.trace("Show file icons in Launchy menus");
			} else {
				launchyShared.trace("Hide file icons in Launchy menus");
			}
		} catch(ex) {
		}

		// cleanup temporary view files
		launchyContext.doSourceCleanup();

		LaunchyContext.progdir = Launchy_DirIO.get("CurProcD").path.toLowerCase();
		// add directory separator but only if we dont end in a separator
		if (!/(\\|\/)$/.test(LaunchyContext.progdir)) {
			LaunchyContext.progdir += Launchy_DirIO.sep;
		}
		launchyShared.trace("Program directory is <" + LaunchyContext.progdir + ">");

		if ("@mozilla.org/winhooks;1" in Components.classes) {
			try {
				LaunchyContext.winhooks = Components.classes["@mozilla.org/winhooks;1"].getService(Components.interfaces.nsIWindowsRegistry);
			} catch(ex) {
				launchyShared.trace("Unable to get winhooks service!");
			}
		} else if ("@mozilla.org/browser/shell-service;1" in Components.classes) {
			try {
				LaunchyContext.winhooks = Components.classes["@mozilla.org/browser/shell-service;1"].getService(Components.interfaces.nsIWindowsShellService);
			} catch(ex) {
				launchyShared.trace("Unable to get browser shell service!");
			}
		} else {
			launchyShared.trace("Unable to get Components. This only works on Windows!");
		}
		if (typeof(LaunchyContext.winhooks) == "undefined" || typeof(LaunchyContext.winhooks.getRegistryEntry) != "function") {
			LaunchyContext.winhooks = false;
			launchyShared.trace("getRegistryEntry isn't available. This only works on Windows!");
		}

		LaunchyContext.apps = new Array();
		LaunchyContext.appsnot = new Array();
		LaunchyContext.appsdisabled = new Array();
		LaunchyContext.appstype = new Array();
		LaunchyContext.appstype[0] = "";
		LaunchyContext.appstype[1] = launchyContext.getBundle("launchy-apps-label-browser", "bundle_launchy_apps");
		LaunchyContext.appstype[2] = launchyContext.getBundle("launchy-apps-label-mail", "bundle_launchy_apps");
		LaunchyContext.appstype[3] = launchyContext.getBundle("launchy-apps-label-media", "bundle_launchy_apps");
		LaunchyContext.appstype[4] = launchyContext.getBundle("launchy-apps-label-ftp", "bundle_launchy_apps");
		LaunchyContext.appstype[5] = launchyContext.getBundle("launchy-apps-label-download", "bundle_launchy_apps");
		LaunchyContext.appstype[6] = launchyContext.getBundle("launchy-apps-label-filemgr", "bundle_launchy_apps");
		LaunchyContext.appstype[7] = launchyContext.getBundle("launchy-apps-label-editor", "bundle_launchy_apps");
		LaunchyContext.appstype[8] = launchyContext.getBundle("launchy-apps-label-image", "bundle_launchy_apps");

		try {
			var disabled = LaunchyContext.pref.getCharPref("extensions.launchy.disabledapps");
			if (disabled) {
				launchyShared.trace("Reading disabledapps pref: " + disabled);
				LaunchyContext.disabledApps = disabled.split(",");
			}
		}	catch(ex) {
			launchyShared.trace("No disabledapps pref");
		}

		if (LaunchyContext.winhooks) {
			launchyShared.trace("Loading applications from JavaScript");

			launchyContext.addApp(2, "Software\\mozilla.org\\Mozilla", "CurrentVersion", "Mozilla", "", 1, "", 1);
			launchyContext.addApp(2, "Software\\mozilla.org\\Mozilla Firebird", "CurrentVersion", "Mozilla Firebird", "", 1, "", 1);
			launchyContext.addApp(2, "Software\\Mozilla\\Mozilla Firefox", "CurrentVersion", "Mozilla Firefox", "", 1, "", 1);
			launchyContext.addApp(2, "Software\\Netscape\\Netscape", "CurrentVersion", "Netscape", "", 1, "", 1);
			launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\IEXPLORE.EXE", "", "Internet Explorer", "", 1, "", 0);
			launchyContext.addApp(2, "Software\\Opera Software", "Last Directory3", "Opera", "opera.exe", 1, "", 0);
			launchyContext.addApp(2, "Software\\Opera Software", "Last Beta Directory", "Opera", "opera.exe", 1, "", 0);
			launchyContext.addApp(2, "Software\\K-Meleon\\K-Meleon\\General", "InstallDir", "K-Meleon", "k-meleon.exe", 1, "", 0);
			launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\Maxthon", "DisplayIcon", "Maxthon", "", 1, "", 0);
			launchyContext.addApp(2, "Software\\Siber Systems\\RoboForm", "_TBP_", "Avant Browser", "", 1, "", 0);
			launchyContext.addApp(2, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\Crazy Browser_is1", "DisplayIcon", "Crazy Browser", "", 1, "", 0);
			launchyContext.addApp(3, "Software\\FlashPeak\\SlimBrowser", "", "Slim Browser", "sbrowser.exe", 1, "", 0);

			launchyContext.addApp(2, "Software\\mozilla.org\\Mozilla", "CurrentVersion", "Mozilla Mail", "", 2, ["-compose"], 1);
			launchyContext.addApp(2, "Software\\Mozilla\\Mozilla Thunderbird", "CurrentVersion", "Mozilla Thunderbird", "", 2, ["-compose"], 1);
			launchyContext.addApp(2, "Software\\Netscape\\Netscape", "CurrentVersion", "Netscape Mail", "", 2, ["-compose"], 1);
			launchyContext.addApp(2, "Software\\Pegasus Mail\\BaseDir", "", "Pegasus Mail", "wsendto.exe", 2, "", 0);
			launchyContext.addApp(3, "Software\\Lotus\\Notes", "Path", "Lotus Notes", "notes.exe", 2, "", 0);
			launchyContext.addApp(2, "Software\\Qualcomm\\Eudora\\CommandLine", "current", "Eudora", "", 2, "", 0);
			launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\OUTLOOK.EXE", "", "Outlook", "", 2, ["-c", "IPM.Note", "/m"], 0);
			launchyContext.addApp(2, "Software\\RIT\\The Bat!", "EXE path", "The Bat!", "", 2, "", 0);
			launchyContext.addApp(3, "Software\\IncrediMail", "InstallationPath", "IncrediMail", "bin\\IncMail.exe", 2, ["/c", "/n", "/mailto:#1"] , 0);
			launchyContext.addApp(3, "Software\\MCS\\Calypso3", "Path", "Calypso", "Calypso.exe", 2, ["/MAILTO"] , 0);
			launchyContext.addApp(3, "Software\\Microsoft\\WAB\\Export\\CSV", "DLL", "Outlook Express", "", 2, ["/mailurl:#1"] , 0, "msimn.exe");
			launchyContext.addApp(2, "Software\\Poco Systems Inc\\PocoMail 3", "Path", "PocoMail", "poco.exe", 2, ["/m"] , 0);
			launchyContext.addApp(2, "Software\\Forte\\Agent\\Paths", "ExeFile", "Forte Agent", "", 2, ["-home:MailIniFile", "-url:#1"] , 0);

			launchyContext.addApp(2, "Software\\BST\\bsplayerv1", "AppPath", "BSPlayer", "", 3, "", 0);
			launchyContext.addApp(3, "Software\\Clients\\Media\\iTunes\\shell\\open\\command", "", "iTunes", "", 3, ["/url"], 0);
			launchyContext.addApp(2, "Software\\Winamp", "", "Winamp", "winamp.exe", 3, "", 0);
			launchyContext.addApp(3, "Software\\Microsoft\\MediaPlayer", "Installation Directory", "Windows Media Player", "wmplayer.exe", 3, "", 0);
			launchyContext.addApp(3, "Software\\Gabest\\Media Player Classic", "ExePath", "Media Player Classic", "", 3, "", 0);
			launchyContext.addApp(0, "Applications\\wordview.exe\\shell\\Open\\command", "", "Word Viewer", "", 3, "", 0);
			launchyContext.addApp(0, "Applications\\xlview.exe\\shell\\Open\\command", "", "Excel Viewer", "", 3, "", 0);

			launchyContext.addApp(2, "Software\\Ghisler\\Total Commander", "InstallDir", "Total Commander", "totalcmd.exe", 4, ["/O"], 0);
			launchyContext.addApp(2, "Software\\UltraFXP", "", "UltraFXP", "ultrafxp.exe", 4, "", 0);
			launchyContext.addApp(3, "Software\\Ipswitch\\WS_FTP", "dir", "WS_FTP Pro", "wsftpgui.exe", 4, "", 0);
			launchyContext.addApp(3, "Software\\SmartFTP", "Install Directory", "SmartFTP", "smartftp.exe", 4, "", 0);
			launchyContext.addApp(2, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\BulletProof FTP_is1", "DisplayIcon", "BulletProof FTP", "", 4, "", 0);
			launchyContext.addApp(2, "Software\\FileZilla", "Install_Dir", "FileZilla", "FileZilla.exe", 4, "", 0);
			launchyContext.addApp(2, "Software\\VanDyke\\AbsoluteFTP\\Install", "Main Directory", "AbsoluteFTP", "AbsoluteFTP.EXE", 4, "", 0);
			launchyContext.addApp(2, "Software\\Altap\\Applications\\Servant Salamander 2.5 beta 7", "LastDirectory", "Servant Salamander", "salamand.exe", 4, ["-R"], 0);

			launchyContext.addApp(2, "Software\\HeadLight\\GetRight\\Config", "InstallDir", "GetRight", "getright.exe", 5, "", 0);
			launchyContext.addApp(2, "Software\\Cronosoft\\LeechGet\\Console", "LeechGetPath", "LeechGet", "", 5, ["-wizard"], 0);
			launchyContext.addApp(0, "TypeLib\\{3F9A5301-9E5A-11D3-A288-0000E80E2EDE}\\1.0\\HELPDIR", "", "Mass Downloader", "massdown.exe", 5, "", 0);
			launchyContext.addApp(2, "Software\\Star Downloader", "Path", "Star Downloader", "", 5, "", 0);
			launchyContext.addApp(2, "Software\\DownloadManager", "ExePath", "Internet Download Manager", "", 5, ["/d"], 0);
			launchyContext.addApp(0, "TypeLib\\{1537E849-0200-11D2-8059-000000000000}\\1.0\\HELPDIR", "", "ReGet Deluxe", "regetdx.exe", 5, "", 0);
			launchyContext.addApp(0, "bittorrent\\shell\\open\\command", "", "BitTorrent", "", 5, ["--url"], 0);
			launchyContext.addApp(2, "Software\\JetCar\\JetCar\\General", "AppPath", "FlashGet", "", 5, "", 0);
			launchyContext.addApp(2, "Software\\Radiate\\Go!Zilla", "", "Go!Zilla", "", 5, "", 0);
			launchyContext.addApp(3, "Software\\SpeedBit\\Download Accelerator", "EXELOCATION", "Download Accelerator", "", 5, "", 0);
			launchyContext.addApp(3, "Software\\Millweed\\WackGet", "", "WackGet", "WackGet.exe", 5, "", 0);
			launchyContext.addApp(2, "Software\\MetaProducts\\Offline Explorer Pro\\Parameters", "LastRunPos", "Offline Explorer Pro", "", 5, ["/new=url=#1"], 0);
			launchyContext.addApp(2, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\NetTransport_is1\\Inno Setup", "App Path", "NetTransport", "", 5, "", 0);

			launchyContext.addApp(2, "Software\\Ghisler\\Total Commander", "InstallDir", "Total Commander", "totalcmd.exe", 6, ["/O"], 0);
			launchyContext.addApp(3, "Software\\Microsoft\\Windows NT\\CurrentVersion", "SystemRoot", "Explorer", "explorer.exe", 6, "", 0);
			launchyContext.addApp(2, "Software\\Altap\\Applications\\Servant Salamander 2.5 beta 7", "LastDirectory", "Servant Salamander", "salamand.exe", 6, "", 0);

			launchyContext.addApp(0, "Applications\\notepad.exe\\shell\\edit\\command", "", "Notepad", "", 7, "", 0);
			launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\UEDIT32.exe", "", "UltraEdit", "", 7, "", 0);
			launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\soffice.exe", "", "OpenOffice.org", "", 7, ["-o"], 0);
			launchyContext.addApp(2, "Software\\mozilla.org\\Mozilla", "CurrentVersion", "Mozilla Composer", "", 7, ["-editor"], 1);
			launchyContext.addApp(2, "Software\\Netscape\\Netscape", "CurrentVersion", "Netscape Composer", "", 7, ["-editor"], 1);
			launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\WORDPAD.EXE", "", "WordPad", "", 7, "", 0);
			launchyContext.addApp(2, "Software\\Ascher\\syn", "", "Syn Text Editor", "", 7, "", 0);
			launchyContext.addApp(0, "TextPad.tws\\DefaultIcon", "", "TextPad", "", 7, "", 0);
			launchyContext.addApp(3, "Software\\Zend Technologies\\Studio", "ZDEPath", "Zend Studio", "", 7, "", 0);
			launchyContext.addApp(0, "CLSID\\{0F0BFAE1-4C90-11D1-82D7-0004AC368519}\\LocalServer32", "", "Vim", "", 7, "", 0);
			launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\PSPad editor_is1", "InstallLocation", "PSPad", "pspad.exe", 7, "", 0);
			launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\Nvu_is1", "InstallLocation", "NVU", "nvu.exe", 7, ["-edit"], 0);
			launchyContext.addApp(2, "Software\\ES-Computing\\EditPlus 2\\Install", "Path", "EditPlus", "editplus.exe", 7, "", 0);

			launchyContext.addApp(3, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\XnView_is1", "Inno Setup: App Path", "XnView", "xnview.exe", 8, "", 0);
			launchyContext.addApp(2, "Software\\Jasc\\Global", "PSP8DefaultRead", "Paint Shop Pro", "Paint Shop Pro.exe", 8, "", 0);
			launchyContext.addApp(2, "Software\\Jasc\\Global", "PSP9DefaultRead", "Paint Shop Pro", "Paint Shop Pro 9.exe", 8, "", 0);
			launchyContext.addApp(0, "Applications\\i_view32.exe\\shell\\open\\command", "", "IrfanView", "", 8, "", 0);

		}
		launchyContext.loadApps();

		w.launchy = LaunchyContext;
	} else {
		LaunchyContext = w.launchy;
		launchyShared.trace("Using cached info from hidden DOM window");
		prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		LaunchyContext.pref = prefService.getBranch(null);
	}
	launchyContext.PrefChangeObserver = {
		observe: function(aSubject, aTopic, aData) {
			launchyContext.prefChanged(aSubject, aTopic, aData);
		}
	};
	try {
		LaunchyContext.pref.addObserver("extensions.launchy", launchyContext.PrefChangeObserver, false);
	} catch(ex) {
		launchyShared.trace("Unable to add addObserver due to:\n" + ex);
	}
}

LaunchyContext.prototype.substWinPaths = function(path) {
	if (LaunchyContext.winhooks) {
		if (/%ProgramFiles%/.test(path)) {
			var ProgramFiles = LaunchyContext.winhooks.getRegistryEntry(3, "Software\\Microsoft\\Windows\\CurrentVersion", "ProgramFilesDir");
			var re=/%ProgramFiles%/;
			path = path.replace(re, ProgramFiles);
		}
		if (/%SystemRoot%/.test(path)) {
			var SystemRoot = LaunchyContext.winhooks.getRegistryEntry(3, "Software\\Microsoft\\Windows NT\\CurrentVersion", "SystemRoot");
			re=/%SystemRoot%/;
			path = path.replace(re, SystemRoot);
		}
		if (/%HOMEDRIVE%/.test(path)) {
			var HOMEDRIVE = LaunchyContext.winhooks.getRegistryEntry(2, "Volatile Environment", "HOMEDRIVE");
			re=/%HOMEDRIVE%/;
			path = path.replace(re, HOMEDRIVE);
		}
		if (/%HOMEPATH%/.test(path)) {
			var HOMEPATH = LaunchyContext.winhooks.getRegistryEntry(2, "Volatile Environment", "HOMEPATH");
			re=/%HOMEPATH%/;
			path = path.replace(re, HOMEDRIVE);
		}
	}
	return path;
}

LaunchyContext.prototype.findApp = function(reg, reg_sub, reg_name, app_name, app_exe, app_gecko, filereplace) {

	// reg values:
	// 0: HKEY_CLASSES_ROOT
	// 1: HKEY_CURRENT_CONFIG
	// 2: HKEY_CURRENT_USER
	// 3: HKEY_LOCAL_MACHINE
	// 4: HKEY_USERS

	var reg_type = new Array("HKEY_CLASSES_ROOT", "HKEY_CURRENT_CONFIG", "HKEY_CURRENT_USER", "HKEY_LOCAL_MACHINE", "HKEY_USERS");
	launchyShared.trace("Looking for <" + app_name + "> in <" + reg_type[reg] + "> at <" + reg_sub + (reg_name?"\\"+reg_name:"") + ">" + (parseInt(app_gecko)?" and it's a Gecko":""));
	try {
		var tmp = LaunchyContext.winhooks.getRegistryEntry(reg, reg_sub, reg_name);
		var tmp_gecko;
		if (app_gecko && tmp) {
			launchyShared.trace("Found Gecko version <" + tmp + ">");
			tmp_gecko = tmp;
			tmp = LaunchyContext.winhooks.getRegistryEntry(reg, reg_sub + "\\" + tmp + "\\Main", "PathToExe");
		}
	} catch(ex) {
		launchyShared.trace("Unable to find <" + app_name + ">");
		return false;
	}
	if (tmp) {
		tmp = launchyContext.substWinPaths(tmp);

		var re=/\\\\/;
		tmp = tmp.replace(re, "\\");
		re=/\\$/;
		tmp = tmp.replace(re, "");
		re=/%1$/;
		tmp = tmp.replace(re, "");
		re=/\s+$/;
		tmp = tmp.replace(re, "");

		launchyShared.trace("<" + app_name + "> registry entry found <" + tmp + ">");
		if (app_exe) {
			tmp += "\\" + app_exe;
		}

		var x;
		// if value contains a comma like: c:\temp\test.exe, 2
		re=/(.*?)?,/;
		x = tmp.match(re);
		if (x && x[1]) {
			launchyShared.trace("Comma removed");
			tmp = x[1];
		}

		// if value is inside quotes like: "c:\temp\test.exe"
		re=/\"(.*?)\" ?/;
		x = tmp.match(re);
		if (x && x[1]) {
			launchyShared.trace("Quotes removed");
			tmp = x[1];
		}

		// if value contains parameters like: c:\temp\test.exe /c
		re=/(.*?) \/.*?$/;
		x = tmp.match(re);
		if (x && x[1]) {
			launchyShared.trace("Parameters removed");
			tmp = x[1];
		}

		// if value contains path paramters like: c:\temp\test.exe c:\temp
		re=/(.*?) [a-zA-Z]:\\/;
		x = tmp.match(re);
		if (x && x[1]) {
			launchyShared.trace("Path paramters removed");
			tmp = x[1];
		}

		if (filereplace) {
			re=/\w+\.\w+$/;
			tmp = tmp.replace(re, filereplace);
		}

		var ft = Launchy_FileIO.open(tmp);
		if (ft && ft.exists()) {
			if (ft.isFile() && ft.isExecutable()) {
				launchyShared.trace(app_name + (app_gecko ? " <" + tmp_gecko + ">" : "") + " is installed at <" + tmp + ">");
				if (app_gecko && tmp.toLowerCase().indexOf(LaunchyContext.progdir) != -1) {
					launchyShared.trace("Hide " + app_name + " since it's the running client");
					return false;
				}
			} else {
				launchyShared.trace("File <" + tmp + "> isn't a file or isn't executable");
				return false;
			}
			return tmp;
		} else {
			launchyShared.trace(app_name + " isn't at: <" + tmp + ">");
			return false;
		}
	} else {
		launchyShared.trace(app_name + " isn't installed or undetected");
		return false;
	}
}

LaunchyContext.prototype.createApp = function(app_name, app_exe, app_type, app_args) {
	var app = new Array();
	app.name = app_name;
	app.exe = app_exe;
	app.type = app_type;
	app.args = app_args;
	return app;
}

LaunchyContext.prototype.addApp = function(reg, reg_sub, reg_name, app_name, app_exe, app_type, app_args, app_gecko, filereplace) {
	// Check for dupes
	for (var i = 0; LaunchyContext.apps.length > i; i++) {
		if (LaunchyContext.apps[i].name == app_name) {
			if (LaunchyContext.apps[i].type != app_type) {
				launchyShared.trace("Reusing <" + app_name + "> detection");
				LaunchyContext.apps[LaunchyContext.apps.length] = launchyContext.createApp(app_name, LaunchyContext.apps[i].exe, app_type, app_args);
				return;
			} else {
				launchyShared.trace("<" + app_name + "> type <" + app_type + "> is already detected");
				return;
			}
		}
	}

	var tmp = launchyContext.findApp(reg, reg_sub, reg_name, app_name, app_exe, app_gecko, filereplace);
	// Check for disabled apps
	if (tmp) {
		if (typeof(LaunchyContext.disabledApps) != "undefined") {
			for (i = 0; i < LaunchyContext.disabledApps.length; ++i) {
				if (LaunchyContext.disabledApps[i] == app_name) {
					LaunchyContext.appsdisabled[LaunchyContext.appsdisabled.length] = launchyContext.createApp(app_name, tmp, app_type, app_args);
					return;
				}
			}
		}
		LaunchyContext.apps[LaunchyContext.apps.length] = launchyContext.createApp(app_name, tmp, app_type, app_args);
	} else {
		LaunchyContext.appsnot[LaunchyContext.appsnot.length] = launchyContext.createApp(app_name, "", app_type, app_args);
	}
}

LaunchyContext.prototype.loadApps = function() {
	var xmlfile;
	if (typeof(LaunchyContext.xmlfilelocation) != "undefined" && LaunchyContext.xmlfilelocation) {
		xmlfile = LaunchyContext.xmlfilelocation;
		launchyShared.trace("Launchy XML file specified");
	} else {
		var chromedir = Launchy_DirIO.get("UChrm");
		chromedir.append("launchy.xml");
		xmlfile = chromedir.path;
	}
	launchyShared.trace("Looking for <" + xmlfile + ">");
	var f = Launchy_FileIO.open(xmlfile);
	if (f && f.exists()) {
		LaunchyContext.xmlfile = xmlfile;
		launchyShared.trace("Loading applications from <" + xmlfile + ">");
		var con = Launchy_FileIO.read(f);
		if (!con) {
			launchyShared.trace("Unable to read file");
			return;
		}
		var domParser=Components.classes["@mozilla.org/xmlextras/domparser;1"].createInstance(Components.interfaces.nsIDOMParser);
		var apps = domParser.parseFromString(con, "text/xml");
		if (apps.firstChild.nodeName == "parsererror") {
			launchyShared.trace("Invalid launchy.xml file:\n"+ apps.firstChild.textContent, 1);
			return;
		}
		var c = apps.getElementsByTagName("configurations");
		if (c) {
			var a = apps.getElementsByTagName("application");
			if (a) {
				for (var i = 0; i < a.length; i++) {
					var tmp;
					var app_name = "";
					tmp = a.item(i).getElementsByTagName("label");
					if (tmp.item(0) && tmp.item(0).firstChild && tmp.item(0).firstChild.nodeValue);
						app_name = tmp.item(0).firstChild.nodeValue;
					var app_type = "";
					tmp = a.item(i).getElementsByTagName("type");
					if (tmp.item(0) && tmp.item(0).firstChild && tmp.item(0).firstChild.nodeValue);
						app_type = tmp.item(0).firstChild.nodeValue;
					var app_exe = "";
					tmp = a.item(i).getElementsByTagName("command");
					if (tmp.item(0) && tmp.item(0).firstChild && tmp.item(0).firstChild.nodeValue) {
						app_exe = tmp.item(0).firstChild.nodeValue;
						if (app_exe)
							app_exe = launchyContext.substWinPaths(app_exe);
					}
					var app_args = "";
					tmp = a.item(i).getElementsByTagName("arguments");
					if (tmp.item(0) && tmp.item(0).firstChild && tmp.item(0).firstChild.nodeValue) {
						app_args = tmp.item(0).firstChild.nodeValue;
						if (app_args)
							app_args = launchyContext.substWinPaths(app_args);
					}
					if (!app_name || !app_type || !app_exe) {
						launchyShared.trace("Invalid launchy.xml file. <label>, <type> and <command> are required.", 1);
						continue;
					}

					var ft = Launchy_FileIO.open(app_exe);
					if (ft && ft.exists() && ft.isFile() && ft.isExecutable()) {
						var dupe = 0;
						for (var k = 0; LaunchyContext.apps.length > k; k++) {
							if (LaunchyContext.apps[k].name == app_name) {
								if (LaunchyContext.apps[k].type != app_type) {
									launchyShared.trace("Reusing <" + app_name + "> detection");
									LaunchyContext.apps[LaunchyContext.apps.length] = launchyContext.createApp(app_name, LaunchyContext.apps[k].exe, app_type, app_args);
									dupe = 1;
									break;
								} else {
									launchyShared.trace("<" + app_name + "> type <" + app_type + "> is already detected");
									dupe = 1;
									break;
								}
							}
						}
						if (!dupe) {
							launchyShared.trace("Found application: <" + app_name + "> type <" + app_type + "> at <" + app_exe + ">" + (app_args ? " with arguments <" + app_args + ">" : ""));
							LaunchyContext.apps[LaunchyContext.apps.length] = launchyContext.createApp(app_name, app_exe, app_type, app_args);
						}
					} else {
						launchyShared.trace("Invalid launchy.xml file. Didn't find application: <" + app_name + "> type <" + app_type + "> at <" + app_exe + "> or it's not executable", 1);
					}
				}
			} else {
				launchyShared.trace("Invalid launchy.xml file. Inner object isn't <application>", 1);
			}
		} else {
			launchyShared.trace("Invalid launchy.xml file. Outer object isn't <configurations>", 1);
		}
	} else if (typeof(LaunchyContext.xmlfilelocation) != "undefined") {
		launchyShared.trace("<" + xmlfile + "> does not exist");
	}
}

LaunchyContext.prototype.getBundle = function(s, i) {
	var b;
	var d = (i ? i : "bundle_launchy");
	try {
		b = document.getElementById(d).getString(s);
	} catch(ex) {
		launchyShared.trace("Unable to get bundle <" + s + "> from <" + d + "> due to:\n" + ex);
	}
	if (!b)
		launchyShared.trace("Unable to get bundle <" + s + "> from <" + d + ">");
	return b;
}

LaunchyContext.prototype.doContext = function(menuid) {
	var children = menuid.childNodes;
	for (var j = children.length-1; j >= 0; j--) {
		menuid.removeChild(children[j]);
	}

	var label;
	var tooltip;
	var flabel;
	var ftooltip;
	var menuitem;

	if (!LaunchyContext.winhooks && typeof(LaunchyContext.apps) == "undefined") {
		launchyShared.trace("No winhooks");
		label = launchyContext.getBundle("launchy-disable-nowinhooks-label");
		tooltip = launchyContext.getBundle("launchy-disable-nowinhooks-tooltip");
		menuitem = document.createElement("menuitem");
		menuitem.setAttribute("label", label);
		menuitem.setAttribute("tooltiptext", tooltip);
		menuitem.setAttribute("oncommand", "launchyShared.help(\"nowinhooks\");");
		menuid.appendChild(menuitem);
		return;
	} else if (typeof(LaunchyContext.apps) == "undefined" || !LaunchyContext.apps.length) {
		launchyShared.trace("No apps");
		label = launchyContext.getBundle("launchy-disable-noapps-label");
		tooltip = launchyContext.getBundle("launchy-disable-noapps-tooltip");
		menuitem = document.createElement("menuitem");
		menuitem.setAttribute("label", label);
		menuitem.setAttribute("tooltiptext", tooltip);
		menuitem.setAttribute("oncommand", "launchyShared.help(\"noapps\");");
		menuid.appendChild(menuitem);
		return;
	}

	var num = 0;
	var sepfound = false;
	// right click on page or click on toolbar
	if (!gContextMenu || !gContextMenu.onLink && !gContextMenu.onMailtoLink && !gContextMenu.onImage) {
		launchyShared.trace("Right click on page or click on toolbar");
		label = launchyContext.getBundle("launchy-page-label");
		tooltip = launchyContext.getBundle("launchy-page-tooltip");
		if (gContextMenu && gContextMenu.inFrame) {
			flabel = launchyContext.getBundle("launchy-page-frame-label");
			ftooltip = launchyContext.getBundle("launchy-page-frame-tooltip");
		}
		for (var i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 1) {
				menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\");");
				if (typeof(LaunchyContext.showfileicons) != "undefined" && LaunchyContext.showfileicons) {
					menuitem.setAttribute("class", "menuitem-iconic");
					menuitem.setAttribute("image", "moz-icon:file://" + LaunchyContext.apps[i].exe + "?size=16");
				}
				menuid.appendChild(menuitem);
				num++;
				if (gContextMenu && gContextMenu.inFrame) {
					menuitem = document.createElement("menuitem");
					menuitem.setAttribute("label", launchyShared.replaceInsert(flabel, 1, LaunchyContext.apps[i].name));
					menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(ftooltip, 1, LaunchyContext.apps[i].name));
					menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 0, 1);");
					if (typeof(LaunchyContext.showfileicons) != "undefined" && LaunchyContext.showfileicons) {
						menuitem.setAttribute("class", "menuitem-iconic");
						menuitem.setAttribute("image", "moz-icon:file://" + LaunchyContext.apps[i].exe + "?size=16");
					}
					menuid.appendChild(menuitem);
				}
			}
		}
		label = launchyContext.getBundle("launchy-source-label");
		tooltip = launchyContext.getBundle("launchy-source-tooltip");
		if (gContextMenu && gContextMenu.inFrame) {
			flabel = launchyContext.getBundle("launchy-source-frame-label");
			ftooltip = launchyContext.getBundle("launchy-source-frame-tooltip");
		}
		for (i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 7) {
				if (sepfound == false && menuid.lastChild && menuid.lastChild.localName != "menuseparator") {
					menuitem = document.createElement("menuseparator");
					menuid.appendChild(menuitem);
					sepfound = true;
				}
				menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doSource(\"" + i + "\", 0, 0);");
				if (typeof(LaunchyContext.showfileicons) != "undefined" && LaunchyContext.showfileicons) {
					menuitem.setAttribute("class", "menuitem-iconic");
					menuitem.setAttribute("image", "moz-icon:file://" + LaunchyContext.apps[i].exe + "?size=16");
				}
				menuid.appendChild(menuitem);
				num++;
				if (gContextMenu && gContextMenu.inFrame) {
					menuitem = document.createElement("menuitem");
					menuitem.setAttribute("label", launchyShared.replaceInsert(flabel, 1, LaunchyContext.apps[i].name));
					menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(ftooltip, 1, LaunchyContext.apps[i].name));
					menuitem.setAttribute("oncommand", "launchyContext.doSource(\"" + i + "\", 0, 1);");
					if (typeof(LaunchyContext.showfileicons) != "undefined" && LaunchyContext.showfileicons) {
						menuitem.setAttribute("class", "menuitem-iconic");
						menuitem.setAttribute("image", "moz-icon:file://" + LaunchyContext.apps[i].exe + "?size=16");
					}
					menuid.appendChild(menuitem);
				}
			}
		}
	}

	// right click on link
	else if (gContextMenu.onLink && gContextMenu.onSaveableLink && !gContextMenu.onMailtoLink) {
		launchyShared.trace("Right click on link");
		label = launchyContext.getBundle("launchy-link-label");
		tooltip = launchyContext.getBundle("launchy-link-tooltip");
		for (i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 1) {
				menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 1);");
				if (typeof(LaunchyContext.showfileicons) != "undefined" && LaunchyContext.showfileicons) {
					menuitem.setAttribute("class", "menuitem-iconic");
					menuitem.setAttribute("image", "moz-icon:file://" + LaunchyContext.apps[i].exe + "?size=16");
				}
				menuid.appendChild(menuitem);
				num++;
			}
		}
		for (i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 3) {
				if (sepfound == false && menuid.lastChild && menuid.lastChild.localName != "menuseparator") {
					menuitem = document.createElement("menuseparator");
					menuid.appendChild(menuitem);
					sepfound = true;
				}
				menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 1);");
				if (typeof(LaunchyContext.showfileicons) != "undefined" && LaunchyContext.showfileicons) {
					menuitem.setAttribute("class", "menuitem-iconic");
					menuitem.setAttribute("image", "moz-icon:file://" + LaunchyContext.apps[i].exe + "?size=16");
				}
				menuid.appendChild(menuitem);
				num++;
			}
		}
		sepfound = false;
		var url = gContextMenu.link.toString();
		launchyShared.trace("Clicked on url <" + url + ">");
		var re=/^(http:\/\/ftp|ftp:\/\/)/i;
		if (url.match(re)) {
			url = url.replace(/^http:\/\//i, "ftp:\/\/");
			LaunchyContext.url = url;
			label = launchyContext.getBundle("launchy-ftp-label");
			tooltip = launchyContext.getBundle("launchy-ftp-tooltip");
			sepfound = false;
			for (i = 0; LaunchyContext.apps.length > i; i++) {
				if (LaunchyContext.apps[i].type == 4) {
					if (sepfound == false && menuid.lastChild && menuid.lastChild.localName != "menuseparator") {
						menuitem = document.createElement("menuseparator");
						menuid.appendChild(menuitem);
						sepfound = true;
					}
					menuitem = document.createElement("menuitem");
					menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
					menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
					menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 1, 0);");
					if (typeof(LaunchyContext.showfileicons) != "undefined" && LaunchyContext.showfileicons) {
						menuitem.setAttribute("class", "menuitem-iconic");
						menuitem.setAttribute("image", "moz-icon:file://" + LaunchyContext.apps[i].exe + "?size=16");
					}
					menuid.appendChild(menuitem);
					num++;
				}
			}
		}
		label = launchyContext.getBundle("launchy-download-label");
		tooltip = launchyContext.getBundle("launchy-download-tooltip");
		sepfound = false;
		for (i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 5) {
				if (sepfound == false && menuid.lastChild && menuid.lastChild.localName != "menuseparator") {
					menuitem = document.createElement("menuseparator");
					menuid.appendChild(menuitem);
					sepfound = true;
				}
				menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 1);");
				if (typeof(LaunchyContext.showfileicons) != "undefined" && LaunchyContext.showfileicons) {
					menuitem.setAttribute("class", "menuitem-iconic");
					menuitem.setAttribute("image", "moz-icon:file://" + LaunchyContext.apps[i].exe + "?size=16");
				}
				menuid.appendChild(menuitem);
				num++;
			}
		}
		if (launchyShared.isLinkType("file", url)) {
			if (/file:\/\/\//.test(url)) {
			} else {
				url = url.replace(/(\w+:\/\/)/, "\\\\");
				launchyShared.trace("Mangled URL <" + url + ">");
				url = url.replace(/(\/)/g, "\\");
				launchyShared.trace("Mangled URL <" + url + ">");
				url = url.replace(/(\+)/g, " ");
				launchyShared.trace("Mangled URL <" + url + ">");
			}
			url = unescape(url);
			launchyShared.trace("Mangled URL <" + url + ">");
			LaunchyContext.url = url;

			label = launchyContext.getBundle("launchy-file-label");
			tooltip = launchyContext.getBundle("launchy-file-tooltip");
			sepfound = false;
			for (i = 0; LaunchyContext.apps.length > i; i++) {
				if (LaunchyContext.apps[i].type == 6) {
					if (sepfound == false && menuid.lastChild && menuid.lastChild.localName != "menuseparator") {
						menuitem = document.createElement("menuseparator");
						menuid.appendChild(menuitem);
						sepfound = true;
					}
					menuitem = document.createElement("menuitem");
					menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
					menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
					menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 1);");
					if (typeof(LaunchyContext.showfileicons) != "undefined" && LaunchyContext.showfileicons) {
						menuitem.setAttribute("class", "menuitem-iconic");
						menuitem.setAttribute("image", "moz-icon:file://" + LaunchyContext.apps[i].exe + "?size=16");
					}
					menuid.appendChild(menuitem);
					num++;
				}
			}
		}
	}

	// right click on mailto link
	else if (gContextMenu.onMailtoLink) {
		launchyShared.trace("Right click on mailto link");
		label = launchyContext.getBundle("launchy-mail-label");
		tooltip = launchyContext.getBundle("launchy-mail-tooltip");
		sepfound = false;
		for (i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 2) {
				menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doApp(\"" + i + "\", 1);");
				if (typeof(LaunchyContext.showfileicons) != "undefined" && LaunchyContext.showfileicons) {
					menuitem.setAttribute("class", "menuitem-iconic");
					menuitem.setAttribute("image", "moz-icon:file://" + LaunchyContext.apps[i].exe + "?size=16");
				}
				menuid.appendChild(menuitem);
				num++;
			}
		}
	}

	// right click on image
	if ((gContextMenu && (gContextMenu.onImage || gContextMenu.onStandaloneImage || gContextMenu.hasBGImage)) || window.content.document.contentType.substr(0,6) == "image/") {
		launchyShared.trace("Right click on image");
		launchyShared.trace(gContextMenu.onImage+":"+gContextMenu.onStandaloneImage+":"+gContextMenu.hasBGImage)
		launchyShared.trace("Image is <" + gContextMenu.imageURL + ">");
		launchyShared.trace("Image is <" + gContextMenu.bgImageURL + ">");
		label = launchyContext.getBundle("launchy-image-label");
		tooltip = launchyContext.getBundle("launchy-image-tooltip");
		sepfound = false;
		if (sepfound == false && menuid.lastChild && menuid.lastChild.localName != "menuseparator") {
			menuitem = document.createElement("menuseparator");
			menuid.appendChild(menuitem);
			sepfound = true;
		}
		for (i = 0; LaunchyContext.apps.length > i; i++) {
			if (LaunchyContext.apps[i].type == 8) {
				menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", launchyShared.replaceInsert(label, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("tooltiptext", launchyShared.replaceInsert(tooltip, 1, LaunchyContext.apps[i].name));
				menuitem.setAttribute("oncommand", "launchyContext.doSource(\"" + i + "\", 0, 0, 1);");
				if (typeof(LaunchyContext.showfileicons) != "undefined" && LaunchyContext.showfileicons) {
					menuitem.setAttribute("class", "menuitem-iconic");
					menuitem.setAttribute("image", "moz-icon:file://" + LaunchyContext.apps[i].exe + "?size=16");
				}
				menuid.appendChild(menuitem);
				num++;
			}
		}
	}

	// add options but only on the toolbar
	if (!gContextMenu) {
		launchyShared.trace("Add option");
		if (menuid.lastChild && menuid.lastChild.localName != "menuseparator") {
			menuitem = document.createElement("menuseparator");
			menuid.appendChild(menuitem);
		}
		menuitem = document.createElement("menuitem");
		menuitem.setAttribute("label", launchyContext.getBundle("launchy-options-label"));
		menuitem.setAttribute("tooltiptext", launchyContext.getBundle("launchy-options-tooltip"));
		menuitem.setAttribute("oncommand", "openDialog('chrome://launchy/content/options.xul', '', 'chrome,modal');");
		if (typeof(LaunchyContext.showfileicons) != "undefined" && LaunchyContext.showfileicons) {
			menuitem.setAttribute("id", "launchy-options-toolbar");
		}
		menuid.appendChild(menuitem);
	}

	// no action
	if (!num) {
		label = launchyContext.getBundle("launchy-disable-noaction-label");
		tooltip = launchyContext.getBundle("launchy-disable-noaction-tooltip");
		menuitem = document.createElement("menuitem");
		menuitem.setAttribute("label", label);
		menuitem.setAttribute("tooltiptext", tooltip);
		menuitem.setAttribute("oncommand", "launchyShared.help(\"noaction\");");
		menuid.appendChild(menuitem);
		return;
	}
}

LaunchyContext.prototype.doApp = function(app_entry, on_link, on_frame) {
	var app_exe = LaunchyContext.apps[app_entry].exe;

	var url;
	if (typeof(LaunchyContext.url) != "undefined" && LaunchyContext.url) {
		url = LaunchyContext.url;
		LaunchyContext.url = "";
	} else {
		if (on_link) {
			url = gContextMenu.link;
		} else {
			if (on_frame) {
				url = window.content.location;
			} else {
				try {
					url = document.commandDispatcher.focusedWindow.location;
				} catch(ex) {
					try {
						url = content.location;
					} catch(ex) {
						try {
							url = gBrowser.contentWindow.location;
						} catch(ex) {
						}
					}
				}
			}
		}
	}

	if (!url) {
		launchyShared.trace("Didn't get an URL");
		return;
	} else {
		launchyShared.trace("Got URL <" + url + ">");
		if (launchyShared.isLinkType("file", url)) {
			url = Launchy_FileIO.urlToPath(url);
			launchyShared.trace("URL converted into <" + url + ">");
		}
	}

	var args = launchyContext.getArgs(LaunchyContext.apps[app_entry], url);

	launchyShared.trace("About to run:\n" + app_exe + " " + args);
	launchyContext.updateStatusbar("Running <" + app_exe + " " + args.join(" ") + ">");

	Launchy_FileIO.run(app_exe, args);
}

LaunchyContext.prototype.updateStatusbar = function(txt) {
	// try to get Mozilla 1.x and Mozilla Firefox statusbar
	var stat = document.getElementById("statusbar-display");
	// try to get Mozilla Thunderbird statusbar
	if (!stat)
		stat = document.getElementById("statusText");
	if (stat)
		stat.label = txt;
}

LaunchyContext.prototype.doSource = function(app_entry, on_link, on_frame, on_image) {
	var app_exe = LaunchyContext.apps[app_entry].exe;

	var url;
	if (on_frame) {
		url = window.content.document;
	} else if (on_link) {
		url = (gContextMenu ? gContextMenu.link : window.content.document);
	} else if (on_image) {
		url = (gContextMenu ? (gContextMenu.imageURL ? gContextMenu.imageURL : gContextMenu.bgImageURL) : window.content.document);
	} else {
		try {
			url = document.commandDispatcher.focusedWindow.document.location;
		} catch(ex) {
			try {
				url = content.document.location;
			} catch(ex) {
				try {
					url = gBrowser.contentWindow.document.location;
				} catch(ex) {
				}
			}
		}
	}

	var tmpfile;
	if (url) {
		launchyShared.trace("URL <" + url + ">");
		if (launchyShared.isLinkType("file", url)) {
			url = url.toString().replace(/[?#].*$/, "");
			launchyShared.trace("URL local <" + url + ">");
			tmpfile = Launchy_FileIO.urlToPath(url);
			launchyShared.trace("Temporary file <" + tmpfile + ">");

		} else {
			var d = new Date().getTime();

			var fe = "html";
			var URLfe = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIURL);
			URLfe.spec = url;
			try {
				if (URLfe.fileExtension) {
					fe = URLfe.fileExtension;
				}
			} catch (ex) {
			}

			var ft = Launchy_DirIO.get("TmpD");
			ft.append("launchy-view-" + d + "." + fe);
			tmpfile = ft.path;

			launchyShared.trace("Temporary file <" + tmpfile + ">");

			var f = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
			f.initWithPath(tmpfile);

			const nsIWBP = Components.interfaces.nsIWebBrowserPersist;
			var persist = Components.classes["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"].createInstance(nsIWBP);
			persist.persistFlags = nsIWBP.PERSIST_FLAGS_FROM_CACHE | nsIWBP.PERSIST_FLAGS_REPLACE_EXISTING_FILES | nsIWBP.PERSIST_FLAGS_DONT_CHANGE_FILENAMES;

			var URI = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIURI);
			URI.spec = url;

			launchyShared.trace("Trying to save URL <" + url + ">");
			try {
				persist.saveURI(URI, null, null, null, null, f);
			} catch (ex) {
				launchyShared.trace("Cannot save URL <" + url + "> due to:\n" + ex);
			}
		}

		var args = launchyContext.getArgs(LaunchyContext.apps[app_entry], tmpfile);

		launchyShared.trace("About to run:\n" + app_exe + " " + args);
		launchyContext.updateStatusbar("Running <" + app_exe + " " + args.join(" ") + ">");

		Launchy_FileIO.run(app_exe, args);
	} else {
		launchyShared.trace("Unable to get URL");
	}
}

LaunchyContext.prototype.getArgs = function(app_entry, url) {
	var xargs = new Array();
	if (app_entry.args && typeof(app_entry.args) == "object") {
		xargs = app_entry.args.join(',').split(',');
		var u = false;
		for (var i=0; i < xargs.length; i++) {
			var tmp = launchyShared.replaceInsert(xargs[i], 1, url);
			if (tmp != xargs[i]) {
				xargs[i] = tmp;
				u = true;
			}
		}
		if (!u) {
			xargs[xargs.length] = url;
		}
	} else if (app_entry.args && typeof(app_entry.args) == "string") {
		tmp = launchyShared.replaceInsert(app_entry.args, 1, url);
		if (tmp != app_entry.args) {
			xargs = tmp.split(" ");
		} else {
			xargs = app_entry.args.split(" ");
			xargs[xargs.length] = url;
		}
	} else {
		xargs[xargs.length] = url;
	}
	return xargs;
}

LaunchyContext.prototype.doSourceCleanup = function() {
	var u = new Date().getTime();
	var tmpdir = Launchy_DirIO.get("TmpD");
	var dir = Launchy_DirIO.open(tmpdir.path);
	var contents = Launchy_DirIO.read(dir, true);
	launchyShared.trace("Number of temp files: <" + contents.length + "> in <" + tmpdir.path + ">");
	var re=/^launchy-view-(\d+)\..*?$/i;
	var u = new Date().getTime();
	for (var i = 0; i < contents.length; i++) {
		if (contents[i].isFile()) {
			var x = contents[i].leafName.match(re);
			if (x && x[1]) {
				// delete temp files older than one day
				if (u - parseInt(x[1]) > 60*60*24*1000) {
					launchyShared.trace("Deleting old view files <" + contents[i].leafName + ">");
					contents[i].remove(false);
				}
			}
		}
	}
	var ua = new Date().getTime();
	launchyShared.trace("Time: <" + ua + "-" + u + "=" + (ua-u) + ">");
}

LaunchyContext.prototype.prefChanged = function(aSubject, aTopic, aData) {
	launchyShared.trace("Pref changed: <" + aData + ">");
	try {
		var pref = LaunchyContext.pref.getBoolPref(aData);
		launchyShared.trace("getpref: " + pref);
	} catch(ex) {
		launchyShared.trace("ex: " + ex);
		pref = false;
		launchyShared.trace("defaultpref: " + pref);
	}
	launchyShared.trace("pref: " + pref);
	switch (aData) {
		case "extensions.launchy.showfileicons":
			launchyShared.trace("ok: extensions.launchy.showfileicons");
			LaunchyContext.showfileicons = pref;
			break;
		case "extensions.launchy.developer":
			launchyShared.trace("ok: extensions.launchy.developer");
			LaunchyContext.developer = pref;
			break;
		case "extensions.launchy.xmlfile":
			launchyShared.trace("xmlfile will be loaded the next time Launchy is loaded");
			break;
		default:
			launchyShared.trace("NOT ok: " + aData);
			break;
	}
}

// create the object
const launchyContext = new LaunchyContext();
window.addEventListener("load", launchyContext.doInit, false);
